create procedure LdbcAddPerson (in personid int,
       		 		       in personfirstname varchar,
				       in personlastname varchar,
				       in gender varchar,
				       in birthday varchar,
				       in creationdate varchar,
				       in locationip varchar,
				       in browserused varchar,
				       in cityid int, 
				       in languages varchar array,
				       in emails varchar array,
				       in tagids varchar array,
				       in studyatorgids varchar array,
				       in studyatyears varchar array,
				       in workatorgids varchar array,
				       in workatyears varchar array
				       )
{
	declare n_dead any;
	n_dead := 0;
	again:	
	declare exit handler for sqlstate '40001' {
		rollback work;
		n_dead := n_dead + 1;
		if (10 < n_dead) {
		   signal ('40001', 'Over 10 deadlocks in rdf load, please retry load');
		   return;
		}
		goto again;
	};
	insert into person values(personid, personfirstname, personlastname, gender,
	       	    	   	  stringdate(birthday), stringdate(creationdate),
				  bit_or(
					bit_or( bit_shift(sprintf_inverse(locationip, '%d.%d.%d.%d', 2)[0], 24),
					       bit_shift(sprintf_inverse(locationip, '%d.%d.%d.%d', 2)[1], 16)),
					bit_or( bit_shift(sprintf_inverse(locationip, '%d.%d.%d.%d', 2)[2], 8),
					       sprintf_inverse(locationip, '%d.%d.%d.%d', 2)[3])
       				  ),
				  browserused, cityid, null);
	--delete from person where p_personid = personid;
	for vectored
	    (in i1 varchar := languages) {
	    insert into person_language values(personid, i1);
	    --delete from person_language where plang_personid = personid;
	}
	for vectored
	    (in i1 varchar := emails) {
	    insert into person_email values(personid, i1);
	    --delete from person_email where pe_personid = personid;
	}
	for vectored
	    (in i1 varchar := tagids) {
	    insert into person_tag values(personid, cast (i1 as bigint));
	    --delete from person_tag where pt_personid = personid;
	}
	for vectored
	    (in i1 varchar := studyatorgids, in i2 varchar := studyatyears) {
	    insert into person_university values(personid, cast (i1 as bigint), cast (i2 as int));
	    --delete from person_university where pu_personid = personid;
	}
	for vectored
	    (in i1 varchar := workatorgids, in i2 varchar := workatyears) {
	    insert into person_company values(personid, cast (i1 as bigint), cast (i2 as int));
	    --delete from person_company where pc_personid = personid;
	}
	commit work;
	return;
};


create procedure add_person (in pers varchar )
{
	-- 17592207534839,"Antonio","Perez","female","1981-07-30","2012-10-20T17:52:49Z","77.75.231.131","Chrome",1355,[],["Antonio17592207534839@gmail.com","Antonio17592207534839@yahoo.com"],[596,1187,1863,9144],[[6336,2001]],[[1192,2006],[1182,2012]]
	declare personid, firstname, lastname, gender, birthday, creationdate, locationip, browserused, placeid any;
	declare ln, ln1, ln2, ln3, ln4, ln5 any;	
	declare i int;
	declare langs, emails any array;
	declare	tagids, u1, u2, c1, c2 any array;
	ln := sprintf_inverse(pers, '%s,"%s","%s","%s","%s","%s","%s","%s",%d,[%s],[%s],[%s],[[%s]],[[%s]]', 0);
	if (isnull(ln))
	   ln := sprintf_inverse(pers, '%s,"%s","%s","%s","%s","%s","%s","%s",%d,[%s],[%s],[%s],[%s],[[%s]]', 0);
	if (isnull(ln))
	   ln := sprintf_inverse(pers, '%s,"%s","%s","%s","%s","%s","%s","%s",%d,[%s],[%s],[%s],[[%s]],[%s]', 0);
	if (isnull(ln))
	   ln := sprintf_inverse(pers, '%s,"%s","%s","%s","%s","%s","%s","%s",%d,[%s],[%s],[%s],[%s],[%s]', 0);
	personid := ln[0];
	firstname := ln[1];
	lastname := ln[2];
	gender := ln[3];
	birthday := ln[4];
	creationdate := ln[5];
	locationip := ln[6];
	browserused := ln[7];
	placeid := ln[8];
	ln1 := split_and_decode (ln[9], 0, '\0\0,');
	langs := make_array (length(ln1), 'any');
	for (i := 0; i < length(ln1); i := i + 1)
	    langs[i] := subseq(ln1[i], 1, length(ln1[i]) - 1);
	ln2 := split_and_decode (ln[10], 0, '\0\0,');
	emails := make_array (length(ln2), 'any');
	for (i := 0; i < length(ln2); i := i + 1)
	    emails[i] := subseq(ln2[i], 1, length(ln2[i]) - 1);
	ln3 := split_and_decode (ln[11], 0, '\0\0,');
	tagids := make_array (length(ln3), 'any');
	for (i := 0; i < length(ln3); i := i + 1)
	    tagids[i] := ln3[i];
	ln[12] := replace(ln[12], '],[', ',');
	ln4 := split_and_decode (ln[12], 0, '\0\0,');
	u1 := make_array (length(ln4)/2, 'any');
	u2 := make_array (length(ln4)/2, 'any');
	for (i := 0; i < length(ln4); i := i + 2)
	{
	    u1[i/2] := ln4[i];
	    u2[i/2] := ln4[i + 1];
	}
	ln[13] := replace(ln[13], '],[', ',');
	ln5 := split_and_decode (ln[13], 0, '\0\0,');
	c1 := make_array (length(ln5)/2, 'any');
	c2 := make_array (length(ln5)/2, 'any');
	for (i := 0; i < length(ln5); i := i + 2)
	{
	    c1[i/2] := ln5[i];
	    c2[i/2] := ln5[i + 1];
	}

	declare state, msg, descs, rows any;
  	state := '00000';
	--dbg_obj_print(forumid, forumtitle);
  	exec ('LdbcAddPerson(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) --Q101', state, msg, vector (cast (personid as int), firstname, lastname, gender, birthday, creationdate, locationip, browserused, cast (placeid as int), langs, emails, tagids, u1, u2, c1, c2), vector ('use_cache', 1), descs, rows);

	if (state <> '00000')
	   signal (state, msg);
	--LdbcAddPerson(personid, firstname, lastname, gender, birthday, creationdate, locationip, browserused, placeid, langs, emails, tagids, u1, u2, c1, c2);
}

create procedure LdbcAddPostLike (in personid int,
       		 			 in postid int,
					 in datetimestr varchar
					 )
{
	declare n_dead any;
	n_dead := 0;
	again:	
	declare exit handler for sqlstate '40001' {
		rollback work;
		n_dead := n_dead + 1;
		if (10 < n_dead) {
		   signal ('40001', 'Over 10 deadlocks in rdf load, please retry load');
		   return;
		}
		goto again;
	};
	insert into likes values(personid, postid, stringdate(datetimestr));
	--delete from likes where l_personid = personid and l_postid = postid;
	commit work;
	return;
};


create procedure add_like_post (in pers varchar )
{
	-- 14293673909528,4398121521961,"2012-10-20T17:48:09Z"
	declare personid, postid, datestr any;
	declare ln any;	
	ln := sprintf_inverse(pers, '%s,%s,"%s"', 0);
	personid := ln[0];
	postid := ln[1];
	datestr := ln[2];

	declare state, msg, descs, rows any;
  	state := '00000';
	--dbg_obj_print(forumid, forumtitle);
  	exec ('LdbcAddPostLike(?, ?, ?) --Q102', state, msg, vector (cast (personid as int), cast (postid as int), datestr), vector ('use_cache', 1), descs, rows);

	if (state <> '00000')
	   signal (state, msg);
	--LdbcAddPostLike(personid, postid, datestr);
}

create procedure LdbcAddCommentLike (in personid int,
       		 			 in postid int,
					 in datetimestr varchar
					 )
{
	declare n_dead any;
	n_dead := 0;
	again:	
	declare exit handler for sqlstate '40001' {
		rollback work;
		n_dead := n_dead + 1;
		if (10 < n_dead) {
		   signal ('40001', 'Over 10 deadlocks in rdf load, please retry load');
		   return;
		}
		goto again;
	};
	insert into likes values(personid, postid, stringdate(datetimestr));
	--delete from likes where l_personid = personid and l_postid = postid;
	commit work;
	return;
};


create procedure add_like_comment (in pers varchar )
{
	-- 14293673909528,4398121521961,"2012-10-20T17:48:09Z"
	declare personid, postid, datestr any;
	declare ln any;	
	ln := sprintf_inverse(pers, '%s,%s,"%s"', 0);
	personid := ln[0];
	postid := ln[1];
	datestr := ln[2];

	declare state, msg, descs, rows any;
  	state := '00000';
	--dbg_obj_print(forumid, forumtitle);
  	exec ('LdbcAddCommentLike(?, ?, ?) --Q103', state, msg, vector (cast (personid as int), cast (postid as int), datestr), vector ('use_cache', 1), descs, rows);

	if (state <> '00000')
	   signal (state, msg);
	--LdbcAddCommentLike(personid, postid, datestr);
}

create procedure LdbcAddForum  (in forumid int,
       		 		       in forumtitle varchar,
				       in creationdate varchar,
				       in moderatorpersonid int, 
				       in tagids varchar array
				       )
{
	declare n_dead any;
	n_dead := 0;
	again:	
	declare exit handler for sqlstate '40001' {
		rollback work;
		n_dead := n_dead + 1;
		if (10 < n_dead) {
		   signal ('40001', 'Over 10 deadlocks in rdf load, please retry load');
		   return;
		}
		goto again;
	};
	insert into forum values(forumid, forumtitle, stringdate(creationdate), moderatorpersonid);
	--delete from forum where f_forumid = forumid;
	for vectored
	    (in i1 varchar := tagids) {
	    insert into forum_tag values(forumid, cast (i1 as bigint));
	    --delete from forum_tag where ft_forumid = forumid;
	}
	commit work;
	return;
};

create procedure add_forum (in pers varchar )
{
	-- 4398049751127,"Group for Maria_Theresa in Afghanistan","2012-10-20T17:48:18Z",17592200662719,[145]
	declare forumid, forumtitle, creationdate, moderatorpersonid any;
	declare	tagids any array;
	declare ln, ln1 any;	
	declare i int;
	ln := sprintf_inverse(pers, '%s,"%s","%s",%s,[%s]', 0);
	forumid := ln[0];
	forumtitle := ln[1];
	creationdate := ln[2];
	moderatorpersonid := ln[3];
	ln1 := split_and_decode (ln[4], 0, '\0\0,');
	tagids := make_array (length(ln1), 'any');
	for (i := 0; i < length(ln1); i := i + 1)
	    tagids[i] := ln1[i];

	declare state, msg, descs, rows any;
  	state := '00000';
	--dbg_obj_print(forumid, forumtitle);
  	exec ('LdbcAddForum(?, ?, ?, ?, ?) --Q104', state, msg, vector (cast (forumid as int), forumtitle, creationdate, cast (moderatorpersonid as int), tagids), vector ('use_cache', 1), descs, rows);

	if (state <> '00000')
	   signal (state, msg);
	--LdbcAddForum(forumid, forumtitle, creationdate, moderatorpersonid, tagids);
}

create procedure LdbcAddForumMembership (in forumid int,
       		 			        in personid int,
						in creationdate varchar
					 	)
{
	declare n_dead any;
	n_dead := 0;
	again:	
	declare exit handler for sqlstate '40001' {
		rollback work;
		n_dead := n_dead + 1;
		if (10 < n_dead) {
		   signal ('40001', 'Over 10 deadlocks in rdf load, please retry load');
		   return;
		}
		goto again;
	};
	insert into forum_person values(forumid, personid, stringdate(creationdate));
	--delete from forum_person where fp_forumid = forumid and fp_personid = personid;
	commit work;
	return;
};


create procedure add_forum_membership (in pers varchar )
{
	-- 3573425429583,6597091821463,"2012-10-20T17:48:09Z"
	declare forumid, personid, creationdate any;
	declare ln any;	
	ln := sprintf_inverse(pers, '%s,%s,"%s"', 0);
	forumid := ln[0];
	personid := ln[1];
	creationdate := ln[2];

	declare state, msg, descs, rows any;
  	state := '00000';
  	exec ('LdbcAddForumMembership(?, ?, ?) --Q105', state, msg, vector (cast (forumid as int), cast (personid as int), creationdate), vector ('use_cache', 1), descs, rows);

	if (state <> '00000')
	   signal (state, msg);
	--LdbcAddForumMembership(forumid, personid, creationdate);
}

create procedure LdbcAddPost (in postid int,
       		 		     in imagefile varchar,
				     in creationdate varchar,
				     in locationip varchar,
				     in browserused varchar,
				     in lang varchar,
				     in content varchar,
				     in len int,
				     in authorpersonid int,
				     in forumid int,
				     in countryid int,
				     in tagids varchar array
				     )
{
	declare n_dead any;
	n_dead := 0;
	again:	
	declare exit handler for sqlstate '40001' {
		rollback work;
		n_dead := n_dead + 1;
		if (10 < n_dead) {
		   signal ('40001', 'Over 10 deadlocks in rdf load, please retry load');
		   return;
		}
		goto again;
	};
	insert into post values(postid, imagefile, stringdate(creationdate),
				  bit_or(
					bit_or( bit_shift(sprintf_inverse(locationip, '%d.%d.%d.%d', 2)[0], 24),
					       bit_shift(sprintf_inverse(locationip, '%d.%d.%d.%d', 2)[1], 16)),
					bit_or( bit_shift(sprintf_inverse(locationip, '%d.%d.%d.%d', 2)[2], 8),
					       sprintf_inverse(locationip, '%d.%d.%d.%d', 2)[3])
       				  ),
				browserused, lang, content, len, authorpersonid, countryid, forumid, NULL, NULL);
	--delete from post where ps_postid = postid;
				
	for vectored
	    (in i1 varchar := tagids) {
	    insert into post_tag values(postid, cast (i1 as bigint));
	    --delete from post_tag where pst_postid = postid;
	}
	commit work;
};

create procedure add_post (in pers varchar )
{
	-- 4398062995240,"photo137439708968.jpg","2012-10-20T17:48:18Z","61.7.134.128","Firefox","","",0,9895612372081,4398062254147,103,[]
	declare postid, imagefile, creationdate, locationip, browserused, lang, content, len, authorpersonid, forumid, countryid any;
	declare	tagids any array;
	declare ln, ln1 any;	
	declare i int;
	ln := sprintf_inverse(pers, '%s,"%s","%s","%s","%s","%s","%s",%s,%s,%s,%s,[%s]', 0);
	postid := ln[0];
	imagefile := ln[1];
	creationdate := ln[2];
	locationip := ln[3];
	browserused := ln[4];
	lang := ln[5];
	content := ln[6];
	len := ln[7];
	authorpersonid := ln[8];
	forumid := ln[9];
	countryid := ln[10];
	ln1 := split_and_decode (ln[11], 0, '\0\0,');
	tagids := make_array (length(ln1), 'any');
	for (i := 0; i < length(ln1); i := i + 1)
	    tagids[i] := ln1[i];

	declare state, msg, descs, rows any;
  	state := '00000';
  	exec ('LdbcAddPost(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) --Q106', state, msg, vector (cast (postid as int), imagefile, creationdate, locationip, browserused, lang, content, cast (len as int), cast (authorpersonid as int), cast (forumid as int), cast (countryid as int), tagids), vector ('use_cache', 1), descs, rows);

	if (state <> '00000')
	   signal (state, msg);
	--LdbcAddPost(postid, imagefile, creationdate, locationip, browserused, lang, content, len, authorpersonid, forumid, countryid, tagids);
}

create procedure LdbcAddComment (in commentid int,
				    	in creationdate varchar,
				     	in locationip varchar,
				     	in browserused varchar,
				     	in content varchar,
				     	in len int,
				     	in authorpersonid int,
				     	in countryid int,
					in replytopostid int,
					in replytocommentid int,
				     	in tagids varchar array
				     	)
{
	declare n_dead any;
	n_dead := 0;
	again:	
	declare exit handler for sqlstate '40001' {
		rollback work;
		n_dead := n_dead + 1;
		if (10 < n_dead) {
		   signal ('40001', 'Over 10 deadlocks in rdf load, please retry load');
		   return;
		}
		goto again;
	};
	insert into post values(commentid, NULL, stringdate(creationdate),
				  bit_or(
					bit_or( bit_shift(sprintf_inverse(locationip, '%d.%d.%d.%d', 2)[0], 24),
					       bit_shift(sprintf_inverse(locationip, '%d.%d.%d.%d', 2)[1], 16)),
					bit_or( bit_shift(sprintf_inverse(locationip, '%d.%d.%d.%d', 2)[2], 8),
					       sprintf_inverse(locationip, '%d.%d.%d.%d', 2)[3])
       				  ),
				browserused, NULL, content, len, authorpersonid, countryid, NULL,
				cast(replytocommentid as bigint) + cast (replytopostid as bigint) +1,
				NULL);
	--delete from post where ps_postid = commentid;				
	for vectored
	    (in i1 varchar := tagids) {
	    insert into post_tag values(commentid, cast (i1 as bigint));
	    --delete from post_tag where pst_postid = commentid;
	}
	commit work;
};


create procedure add_comment (in pers varchar )
{
	-- 4398185298077,"2012-10-20T17:48:09Z","199.7.168.242","Firefox","right",5,13194148142309,36,-1,4398185298070,[]
	declare commentid, creationdate, locationip, browserused, content, len, authorpersonid, countryid, replytopostid, replytocommentid any;
	declare	tagids any array;
	declare ln, ln1 any;	
	declare i int;
	ln := sprintf_inverse(pers, '%s,"%s","%s","%s","%s",%s,%s,%s,%s,%s,[%s]', 0);
	commentid := ln[0];
	creationdate := ln[1];
	locationip := ln[2];
	browserused := ln[3];
	content := ln[4];
	len := ln[5];
	authorpersonid := ln[6];
	countryid := ln[7];
	replytopostid := ln[8];
	replytocommentid := ln[9];
	ln1 := split_and_decode (ln[10], 0, '\0\0,');
	tagids := make_array (length(ln1), 'any');
	for (i := 0; i < length(ln1); i := i + 1)
	    tagids[i] := ln1[i];

	declare state, msg, descs, rows any;
  	state := '00000';
  	exec ('LdbcAddComment(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) --Q107', state, msg, vector (cast (commentid as int), creationdate, locationip, browserused, content, cast (len as int), cast (authorpersonid as int), cast (countryid as int), cast (replytopostid as int), cast (replytocommentid as int), tagids), vector ('use_cache', 1), descs, rows);

	if (state <> '00000')
	   signal (state, msg);
	--LdbcAddComment(commentid, creationdate, locationip, browserused, content, len, authorpersonid, countryid, replytopostid, replytocommentid, tagids);
}

create procedure LdbcAddFriendship (in person1id int,
       		 			   in person2id int,
					   in creationdate varchar
					   )
{
	declare n_dead any;
	n_dead := 0;
	again:	
	declare exit handler for sqlstate '40001' {
		rollback work;
		n_dead := n_dead + 1;
		if (10 < n_dead) {
		   signal ('40001', 'Over 10 deadlocks in rdf load, please retry load');
		   return;
		}
		goto again;
	};
	insert into knows values(person1id, person2id, stringdate(creationdate));
	--delete from knows where k_person1id = person1id and k_person2id = person2id;
	commit work;
	return;
};

create procedure add_friendship (in pers varchar )
{
	-- 14293657902499,17592192585404,"2012-10-20T17:48:21Z"
	declare person1id, person2id, creationdate any;
	declare ln any;	
	ln := sprintf_inverse(pers, '%s,%s,"%s"', 0);
	person1id := ln[0];
	person2id := ln[1];
	creationdate := ln[2];

	declare state, msg, descs, rows any;
  	state := '00000';
  	exec ('LdbcAddFriendship(?, ?, ?) --Q108', state, msg, vector (cast (person1id as int), cast (person2id as int), creationdate), vector ('use_cache', 1), descs, rows);

	if (state <> '00000')
	   signal (state, msg);
	--LdbcAddFriendship(person1id, person2id, creationdate);
}


create procedure snb_load_1 (in fn varchar )
{

  declare f any;
  declare line any;
  declare ln any;
  declare i int;
  i := 0;
  log_enable (2, 1);
  set non_txn_insert = 1;
  f := file_open (fn);

  while (line := file_read_line (f))
    {
      ln := split_and_decode (line, 0, '\0\0|');
      ln[2] := subseq(ln[2], 1, length(ln[2]) - 1);
      if (ln[1] like 'ADD_PERSON')
            add_person(ln[2]);
      if (ln[1] like 'ADD_LIKE_POST')
            add_like_post(ln[2]);
      if (ln[1] like 'ADD_LIKE_COMMENT')
            add_like_post(ln[2]);
      if (ln[1] like 'ADD_FORUM')
            add_forum(ln[2]);
      if (ln[1] like 'ADD_FORUM_MEMBERSHIP')
            add_forum_membership(ln[2]);
      if (ln[1] like 'ADD_POST')
            add_post(ln[2]);
      if (ln[1] like 'ADD_COMMENT')
            add_comment(ln[2]);
      if (ln[1] like 'ADD_FRIENDSHIP')
            add_friendship(ln[2]);
      continue_loop:
      i := i + 1;
    }
    return i;
}

--snb_load_1('/home/mirko/updates/part00') &
--snb_load_1('/home/mirko/updates/part01') &
--snb_load_1('/home/mirko/updates/part02') &
--snb_load_1('/home/mirko/updates/part03') &
--snb_load_1('/home/mirko/updates/part04') &
--snb_load_1('/home/mirko/updates/part05') &
--snb_load_1('/home/mirko/updates/part06') &
--snb_load_1('/home/mirko/updates/part07') &
--snb_load_1('/home/mirko/updates/part08') &
--snb_load_1('/home/mirko/updates/part09') &
--snb_load_1('/home/mirko/updates/part10') &
--snb_load_1('/home/mirko/updates/part11') &
--snb_load_1('/home/mirko/updates/part12') &
--snb_load_1('/home/mirko/updates/part13') &
--snb_load_1('/home/mirko/updates/part14') &
--snb_load_1('/home/mirko/updates/part15') &
--
--wait_for_children;
--


